<?php 

namespace Naiz\NaizFitSizeForm\Model;
use Naiz\NaizFitSizeForm\Api\FeedInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Model\ResourceModel\Category;
use Magento\Framework\App\RequestInterface;

class Feed implements FeedInterface {

  private $productRepository;
  private $searchCriteriaBuilder;
  private $sortOrderBuilder;
  private $categoryCollectionFactory;
  private $storeManager;

  public function __construct(
      ProductRepositoryInterface $productRepository,
      SearchCriteriaBuilder $searchCriteriaBuilder,
      SortOrderBuilder $sortOrderBuilder,
      Category\CollectionFactory $categoryCollectionFactory,
      StoreManagerInterface $storeManager,
      RequestInterface $request
  ) {
      $this->productRepository = $productRepository;
      $this->searchCriteriaBuilder = $searchCriteriaBuilder;
      $this->sortOrderBuilder = $sortOrderBuilder;
      $this->categoryCollectionFactory = $categoryCollectionFactory;
      $this->storeManager = $storeManager;
      $this->request = $request;
  }
  
  public function getFeed() {
    $count = $this->request->has("count") ? $this->request->getParam("count") : 200;
    $page = $this->request->has("page") ? $this->request->getParam("page") : 1;
    $searchCriteria = $this->searchCriteriaBuilder->create();
    $searchCriteria->setPageSize($count)->setCurrentPage($page);
    $products = $this->productRepository->getList($searchCriteria)->getItems();
    $stores2 = $this->storeManager->getStores();
    $stores = array();
    foreach ($stores2 as $store) {
      if ($store->IsActive()) $stores[] = $store;
    }
    return array_map(function($product) use ($stores) {
      $banned_attributes = array ("meta_description","description","page_layout","custom_design","custom_layout","gift_message_available","options_container","price_view","media_gallery","ts_packaging_type","ts_packaging_id","quantity_and_stock_status","shipment_type","msrp_display_actual_price_type","visibility");

      $categoriesFac = $this->categoryCollectionFactory->create();
      $categoriesFac->addAttributeToSelect('*');        
      
      $options = $product->getTypeId() == "configurable" ? array_merge(... $product->getTypeInstance()->getConfigurableOptions($product)) : array();
      $options = array_reduce(array_map(function($option) {
        return array("code" => $option["attribute_code"], "title" => $option["option_title"]);
      }, $options), function($carry, $item) { 
        if(!in_array($item, $carry))
        $carry[] = $item;
        return $carry;
      }, array());
      
      $attributes = $product->getAttributes();
      $attributes2 = array();
      foreach ($attributes as $attribute){
        $value = $attribute->getFrontend()->getValue($product);
        $code = $attribute->getAttributeCode();
        if ($value && !in_array($code,$banned_attributes)) $attributes2[$code] = $value;
      }
      
      $categoryIds = $product->getCategoryIds();
      $categories2 = array();
      if (count($categoryIds) > 0){
        $categories = $categoriesFac->addAttributeToFilter('entity_id', $categoryIds);  
        $categories = $categories->getItems();
        foreach ($categories as $category) {
          if ($category->getIsActive() != 0) $categories2[] = array ( "code" => $category->getId(), "parent_code" => $category->getParentId(), "path" => $category->getPath(), "position" => $category->getPosition(), "level" => $category->getLevel(), "name" => $category->getName(), "active" => $category->getIsActive(),"url_key"=>$category->getUrlKey(),"url_path"=>$category->getUrlPath());
        }
      }

      $urls = array();
      foreach ($stores as $store){
        $urls[] = $product->setStoreId($store->getId())->getUrlModel()->getUrlInStore($product);
      }

      $price = $product->getTypeId() == "configurable" ? $product->getPriceInfo()->getPrice('regular_price')->getMaxRegularAmount()->getValue() : $product->getPrice();
      return array(
        "name" => $product->getName(),
        "sku" => $product->getSku(),
        "url" => $product->getUrlModel()->getUrl($product),
        "other_urls" => $urls,
        "price" => $product->getPrice(),
        "status" => $product->getStatus(),
        "categoryIds" => $product->getCategoryIds(),
        "categories" => $categories2,
        "qty" => $product->getQty(),
        "price" => ($price),
        "inStock" => $product->isInStock(),
        "images" => array_map(function($image) {return $image->getUrl();}, $product->getMediaGalleryImages()->getItems()),
        "options" => $options,
        "attributes" => $attributes2
      );
    }, $products);
	}
}